package main

import (
	"flag"
	"fmt"
	"log"
	"os"
)

type CommandFunc func(tbm *TBM, args []string) error

var (
	serial_device   = flag.String("serial-device", "/dev/ttyUSB0", "Serial device to use")
	serial_baudrate = flag.Int("serial-baud", 9600, "Serial baud rate")

	commands = map[string]CommandFunc{
		"ls": cmd_ls,
		"cat": cmd_cat,
		"stat": cmd_stat,
		"cp": cmd_cp,
		"rm": cmd_rm,
		"mkdir": cmd_mkdir,
		"rmdir": cmd_rmdir,
		"date": cmd_date,
		"set-date": cmd_set_date,
		"time": cmd_time,
		"write": cmd_write,
		"append": cmd_append,
	}
)

func cmd_generic(tbm *TBM, command string, args []string, payload []byte) error {
	cmd := &Command{command, args, payload}
	tbm.Commands <- cmd
	res := <-tbm.Results

	if res.errcode[0] != '0' {
		return fmt.Errorf("Error: %s", res.errcode)
	}
	fmt.Printf("%s", res.output)

	return nil
}

func cmd_ls(tbm *TBM, args []string) error {
	return cmd_generic(tbm, "mufs ls", args, nil)
}

func cmd_cat(tbm *TBM, args []string) error {
	return cmd_generic(tbm, "mufs cat", args, nil)
}

func cmd_stat(tbm *TBM, args []string) error {
	return cmd_generic(tbm, "mufs stat", args, nil)
}

func cmd_cp(tbm *TBM, args []string) error {
	return cmd_generic(tbm, "mufs cp", args, nil)
}

func cmd_rm(tbm *TBM, args []string) error {
	return cmd_generic(tbm, "mufs rm", args, nil)
}

func cmd_mkdir(tbm *TBM, args []string) error {
	return cmd_generic(tbm, "mufs mkdir", args, nil)
}

func cmd_rmdir(tbm *TBM, args []string) error {
	return cmd_generic(tbm, "mufs rmdir", args, nil)
}

func cmd_date(tbm *TBM, args []string) error {
	return cmd_generic(tbm, "date", args, nil)
}

func cmd_set_date(tbm *TBM, args []string) error {
	return cmd_generic(tbm, "set-date", args, nil)
}

func cmd_time(tbm *TBM, args []string) error {
	return cmd_generic(tbm, "time", args, nil)
}

func cmd_write(tbm *TBM, args []string) error {
	f, _ := os.Open(args[0])
	defer f.Close()

	buf := make([]byte, 256)

	n, err := f.Read(buf)

	if n == 0 {
		return err
	}

	cmd_generic(tbm, "mufs write", []string{args[1]}, buf)

	for {
		n, err = f.Read(buf)

		if n == 0 {
			return err
		}

		cmd_generic(tbm, "mufs append", []string{args[1]}, buf)
	}
}

func cmd_append(tbm *TBM, args []string) error {
	var n int
	var err error

	f, _ := os.Open(args[0])
	defer f.Close()

	buf := make([]byte, 256)

	for {
		n, err = f.Read(buf)

		if n == 0 {
			return err
		}

		cmd_generic(tbm, "mufs append", []string{args[1]}, buf)
	}
}

func main() {
	flag.Parse()

	args := flag.Args()
	if len(args) == 0 {
		log.Fatalf("Please specify at least one command")
	}
	command := args[0]

	command_function, ok := commands[command]

	if !ok {
		log.Fatalf("Unknown command: %s", command)
	}

	t := &TBM{}
	err := t.Connect(*serial_device, *serial_baudrate)
	if err != nil {
		log.Fatal(err)
	}
	t.Handle()
	defer t.Close()

	err = command_function(t, args[1:])
	if err != nil {
		log.Fatal(err)
	}
}
